unit libiprivheader;

interface
{*
   Copyright (C) 1998-2016 CyberPlat. All Rights Reserved.
   e-mail: tech@cyberplat.com
*}

uses SysUtils, windows;

const
 SLibiprivDll = 'libipriv.dll';

// 
 IPRIV_ENGINE_RSAREF    = 0; //  RSAREF
 IPRIV_ENGINE_OPENSSL   = 1; //  OpenSSL
 IPRIV_ENGINE_PKCS11    = 2; //  PKCS11 (  eToken)
 IPRIV_ENGINE_WINCRYPT  = 3; //  Microsoft Windows CryptoAPI
 IPRIV_ENGINE_SENSELOCK = 4; //  SenseLock

//  
 IPRIV_ENGCMD_IS_READY              = 0; // in: none, retval: 1-ready, 0 - not ready
 IPRIV_ENGCMD_GET_ERROR             = 1; // in: none, retval: errcode
 IPRIV_ENGCMD_SET_PIN               = 2; // in: const char* null-terminated pin code, retval: 0-success
 IPRIV_ENGCMD_SET_PKCS11_LIB        = 3; // in: static const char* null-terminated path to library, retval: 0-success
 IPRIV_ENGCMD_GET_PKCS11_SLOTS_NUM  = 4; // in: none, retval - slots num or 0
 IPRIV_ENGCMD_GET_PKCS11_SLOT_NAME	= 5; // in: int slot index, char* dst, int ndst, retval - string length
 IPRIV_ENGCMD_SET_PKCS11_SLOT       = 6; // in: int slot index (from 0), retval - 0-success
 IPRIV_ENGCMD_ENUM_PKCS11_KEYS      = 7; // in: IPRIV_KEY* array, int array max size, retval - keys num
 IPRIV_ENGCMD_ENUM_PKCS11_PUBKEYS   = 8; // in: IPRIV_KEY* array, int array max size, retval - keys num
 IPRIV_ENGCMD_PEM_EXPORT            = 9; // in: IPRIV_KEY* key, char *passphrase, char *dst, int ndst, retval: 0 success
 IPRIV_ENGCMD_PEM_IMPORT            = 10; // in: IPRIV_KEY* key, char *passphrase, char *src, int nsrc, retval: 0 success
 IPRIV_ENGCMD_GET_KEY_LENGTH        = 11; // in: IPRIV_KEY* key, retval: key length
 IPRIV_ENGCMD_SET_PIN2              = 12; // in: const char* null-terminated pin code, retval: 0-success
 IPRIV_ENGCMD_PKCS11_DELKEY         = 13;  // in: unsigned long keyserial, retval: 0 - success

//  
 CRYPT_ERR_BAD_ARGS              = -1; //   
 CRYPT_ERR_OUT_OF_MEMORY         = -2; //   
 CRYPT_ERR_INVALID_FORMAT        = -3; //   
 CRYPT_ERR_NO_DATA_FOUND         = -4; //     
 CRYPT_ERR_INVALID_PACKET_FORMAT = -5; //     
 CRYPT_ERR_UNKNOWN_ALG           = -6; //   
 CRYPT_ERR_INVALID_KEYLEN        = -7; //      
 CRYPT_ERR_INVALID_PASSWD        = -8; //     
 CRYPT_ERR_DOCTYPE               = -9; //   
 CRYPT_ERR_RADIX_DECODE          = -10; //  ASCII  
 CRYPT_ERR_RADIX_ENCODE          = -11; //  ASCII  
 CRYPT_ERR_INVALID_ENG           = -12; //   
 CRYPT_ERR_ENG_NOT_READY         = -13; //   
 CRYPT_ERR_NOT_SUPPORT           = -14; //    
 CRYPT_ERR_FILE_NOT_FOUND        = -15; //   
 CRYPT_ERR_CANT_READ_FILE        = -16; //   
 CRYPT_ERR_INVALID_KEY           = -17; //     
 CRYPT_ERR_SEC_ENC               = -18; //   
 CRYPT_ERR_PUB_KEY_NOT_FOUND     = -19; //       
 CRYPT_ERR_VERIFY                = -20; //     
 CRYPT_ERR_CREATE_FILE           = -21; //   
 CRYPT_ERR_CANT_WRITE_FILE       = -22; //    
 CRYPT_ERR_INVALID_KEYCARD       = -23; //    
 CRYPT_ERR_GENKEY                = -24; //   
 CRYPT_ERR_PUB_ENC               = -25; //  
 CRYPT_ERR_SEC_DEC               = -26; //  
 CRYPT_ERR_UNKNOWN_SENDER        = -27; //  
 
 //  
IPRIV_ALG_MD5 = 1;
IPRIV_ALG_SHA256 = 2;

 //  
type
 TIprivKey = packed record //packed array [0..47] of byte; //     ,     
              eng: smallint;
              key_type: smallint;
              key_serial: dword;
              owner: array [0..23] of char;
              data_ptr: pointer;
              flags: dword;
              timestamp: dword;
             end;
 PIprivKey = ^TIprivKey;

// ************************
//     *
// ************************

procedure Crypt_Start;

procedure Crypt_Stop;

//  .
//         (  ).
// : 0 -    
function Crypt_Initialize: integer; stdcall; external SLibiprivDll;


//  .
//         (  ).
// : 0 -    
function Crypt_Done: integer; stdcall; external SLibiprivDll;

//       Crypt_Sign*
function Crypt_SetHashAlg(alg: integer): integer; stdcall; external SLibiprivDll;

//        (Crypt_OpenSecretKey),   (Crypt_OpenSecretKeyFromFile)
//      (Crypt_OpenSecretKeyFromStore). Crypt_OpenSecretKey2  
//     (    base64).
// eng: ,  
// src: ,     
// nsrc: ,  , -1 -   (  -)
// path: ,      
// passwd: ,      
// keyserial: ,    
// key: ,  
// : 0 -    
function Crypt_OpenSecretKey(eng: integer; const src: pchar; nsrc: integer; const passwd: pchar; var key: TIprivKey): integer; stdcall; external SLibiprivDll;
function Crypt_OpenSecretKeyFromFile(eng: integer; const path: pchar; const passwd: pchar; var key: TIprivKey): integer; stdcall; external SLibiprivDll;
function Crypt_OpenSecretKeyFromStore(eng: integer; keyserial: dword; var key: TIprivKey): integer; stdcall; external SLibiprivDll;

//    (   ).
// dst: ,     
// ndst: ,    
// path: ,      
// passwd: ,      
// key: ,  
// :      

function Crypt_ExportSecretKey(dst: PChar; ndst: integer;const passwd: PChar; key: TIprivKey): integer; stdcall; external SLibiprivDll;
function Crypt_ExportSecretKeyToFile(const path: PChar; const passwd: PChar; key: TIprivKey): integer; stdcall; external SLibiprivDll;

//        (  ).
// ,        eToken.
// eng: ,  
// src: ,     
// nsrc: ,  , -1 -   (  -)
// path: ,      
// passwd: ,      
// : 0 -    

function Crypt_ImportSecretKey(eng: integer; const src: PChar; nsrc: integer; const passwd: PChar): integer; stdcall; external SLibiprivDll;
function Crypt_ImportSecretKeyFromFile(eng: integer; const path: PChar; const passwd: PChar): integer; stdcall; external SLibiprivDll;

//        (Crypt_OpenPublicKey),   (Crypt_OpenPublicKeyFromFile)
//      (Crypt_OpenPublicKeyFromStore).
// eng: ,  
// src: ,     
// nsrc: ,  , -1 -   (  -)
// path: ,      
// keyserial: ,    
// key: ,  
// cakey: ,   0,      
// : 0 -    

function Crypt_OpenPublicKey(eng: integer; const src: PChar; nsrc: integer; keyserial: longword; key, cakey: TIprivKey): integer; stdcall external SLibiprivDll;
function Crypt_OpenPublicKeyFromFile(eng: integer; const path: Pchar; keyserial: longword; key, cakey: TIprivKey): integer; stdcall external SLibiprivDll;
function Crypt_OpenPublicKeyFromStore(eng: integer; keyserial: longword; key: TIprivKey): integer; stdcall; external SLibiprivDll;

//   .
// dst: ,     
// ndst: ,    
// path: ,      
// key: ,  
// cakey: ,   0,       
// :      

function Crypt_ExportPublicKey(dst: PChar; ndst: integer; key, cakey: TIprivKey):integer; stdcall; external SLibiprivDll;
function Crypt_ExportPublicKeyToFile(const path: PChar; key, cakey: TIprivKey):integer; stdcall; external SLibiprivDll;

//        (  ).
// ,        eToken.
// eng: ,  
// src: ,     
// nsrc: ,  , -1 -   (  -)
// path: ,      
// keyserial: ,    
// cakey: ,   0,      
// : 0 -    

function Crypt_ImportPublicKey(eng: integer; const src: PChar; nsrc: integer; keyserial: longword; cakey: TIprivKey): integer; stdcall; external SLibiprivDll;
function Crypt_ImportPublicKeyFromFile(eng: integer; const path: PChar; keyserial: longword; cakey: TIprivKey): integer; stdcall; external SLibiprivDll;

//   .
// src: ,    
// nsrc:  , -1 -   (  -)
// dst: ,      
// ndst: ,    
// key: ,  
// :      
function Crypt_Sign(const src: pchar; nsrc: integer; dst: Pchar; ndst: integer; var key: TIprivKey): integer; stdcall; external SLibiprivDll;
function Crypt_SignEx(const src: pchar; nsrc: integer; dst: Pchar; ndst: integer; var key: TIprivKey; alg: integer): integer; stdcall; external SLibiprivDll;

//     .
// src: ,    
// nsrc:  , -1 -   (  -)
// dst: ,     
// ndst: ,    
// key: ,  
// alg: hash 
// :      
function Crypt_Sign2(const src: pchar; nsrc: integer; dst: Pchar; ndst: integer; var key: TIprivKey): integer; stdcall; external SLibiprivDll;
function Crypt_Sign2Ex(const src: pchar; nsrc: integer; dst: Pchar; ndst: integer; var key: TIprivKey; alg: integer): integer; stdcall; external SLibiprivDll;

//   .
// src: ,    
// nsrc:  , -1 -   (  -)
// pdst: ,   0,  ,       ( )
// pndst: ,   0,  ,       ( )
// key: ,  
// : 0 -    
function Crypt_Verify(const src: PChar; nsrc: integer; const pdst: PPChar; pndst: PInteger; key: TIprivKey): integer; stdcall external SLibiprivDll;

//     .
// src: ,    
// nsrc:  , -1 -   (  -)
// pdst: ,   0,  ,       ( )
// pndst: ,   0,  ,       ( )
// key: ,  
// : 0 -    
function Crypt_Verify_Detached(const src: PChar; nsrc: integer; const pdst: PPChar; pndst: PInteger; key: TIprivKey): integer; stdcall external SLibiprivDll;

//   .       .
// src: ,    
// nsrc:  , -1 -   (  -)
// dst: ,     
// ndst: ,    
// :      
function Crypt_Encrypt(const src: PChar; nsrc: integer; dst: PChar; ndst: integer; key: TIprivKey):integer; stdcall; external SLibiprivDll;
function Crypt_EncryptLong(const src: PChar; nsrc: integer; dst: PChar; ndst: integer; key: TIprivKey):integer; stdcall; external SLibiprivDll;

//   .
// src: ,    
// nsrc:   , -1 -   (  -)
// dst: ,    
// ndst: ,    
// :     
function Crypt_Decrypt(const src: pchar; nsrc: integer; dst: pchar; ndst: integer; var key: TIprivKey): integer; stdcall; external SLibiprivDll;
function Crypt_DecryptLong(const src: pchar; nsrc: integer; dst: pchar; ndst: integer; var key: TIprivKey): integer; stdcall; external SLibiprivDll;

//  .
// key: ,    
// : 0 -    
function Crypt_CloseKey(var key: TIprivKey): integer; stdcall; external SLibiprivDll;

//     
function Crypt_GetKeyBits(var key: TIprivKey): integer; stdcall; external SLibiprivDll;

//    .
//       .
// ,  -      eToken.
// eng: ,  
// cmd: ,  
// :    
function Crypt_Ctrl_Null(eng: integer; cmd: integer): integer; stdcall; external SLibiprivDll;
function Crypt_Ctrl_String(eng: integer; cmd: integer; txt: pchar): integer; stdcall; external SLibiprivDll;
function Crypt_Ctrl_Int(eng: integer; cmd: integer; var buf; szbuf: integer): integer; stdcall; external SLibiprivDll;
function Crypt_Ctrl_Ptr(eng: integer; cmd: integer; arg: pointer): integer; stdcall; external SLibiprivDll;

//      (Crypt_GenKeyCard)    (Crypt_GenKeyCardToFile).
// dst: ,      
// ndst: ,    
// path: ,      
// userid: ,  
// keyserial: ,   
// :      

function Crypt_GenKeyCard(dst: PChar; ndst: integer; const userid: PChar; keyserial: longword): integer; stdcall; external SLibiprivDll;
function Crypt_GenKeyCardToFile(const path: PChar; const userid: PChar; keyserial: longword): integer; stdcall; external SLibiprivDll;

//   
// path: ,      
// keyserial: ,   
// userid: ,  
// : 0 -    

function Crypt_ReadKeyCardFromFile(const path: PChar; keyserial: longword; userid: PChar): integer;  stdcall; external SLibiprivDll;

//    (/)    .
// eng: ,  
// src: ,     
// nsrc: ,  , -1 -   (  -)
// keycardpath: ,      
// sec: ,  
// pub: ,  
// bits: ,    
// : 0 -    

function Crypt_GenKey(eng: integer; const src: PChar; nsrc: integer; sec: TIprivKey; pub: TIprivKey; bits: integer): integer; stdcall; external SLibiprivDll;
function Crypt_GenKeyFromFile(eng: integer; const keycardpath: Pchar; sec: TIprivKey; pub: TIprivKey; bits: integer): integer; stdcall; external SLibiprivDll;
function Crypt_GenKey2(eng: integer; keyserial: longword; const userid: PChar; sec: TIprivKey; pub: TIprivKey; bits: integer): integer; stdcall; external SLibiprivDll;

implementation

var
 crypto_init_count: integer;

procedure Crypt_Start;
var
 res: integer;
begin
 if crypto_init_count = 0 then
  begin
   res := Crypt_Initialize;
   if res <> 0 then raise Exception.CreateFmt('    L: %d',[res]);
  end;
 inc(crypto_init_count);
end;

procedure Crypt_Stop;
begin
 dec(crypto_init_count);
 if crypto_init_count = 0 then Crypt_Done;
end;

initialization
 crypto_init_count := 0;

finalization
 if crypto_init_count > 0 then Crypt_Done;
end.
